# InvAxiomCalc v1.0 #

The system InvAxiomCalc takes as input a Hilbert-style axiom and transforms
it, if possible, into an equivalent set of sequent or hypersequent rules according
to the algorithm in [1].


### Requirements ###

In order to run InvAxiomCalc, the following packages need to be installed:

  * The sources of the latest version of InvAxiomCalc.
  * A stable Prolog environment e.g. [SWI-Prolog](http://www.swi-prolog.org/).
    InvAxiomCalc has been developed and tested with SWI-Prolog version 7.4.2.

### Using InvAxiomCalc ###

Follow these steps to run InvAxiomCalc:

 1. Open a terminal and navigate to the root folder of the InvAxiomCalc sources.
 2. Enter the prolog environment. In SWI-Prolog this is done by the command:

           $ swipl

 3. Consult the file `invac_kernel.pl` (which is the entry point of InvAxiomCalc)
 by typing the command:

           ?- [invac_kernel].

 4. Type the command `compute.` and then enter the input axiom to be transformed
 followed by a dot `.` e.g:

           ?- compute.
           |: -a + (a * a).


  Read the next section for the syntax and the conventions of input axioms.
  As a result of this command, the following output will be displayed in
  terminal:

	This axiom is in the class: n(2)

	Equivalent Analytic Rule:

	 |- ['Y'+1,'Y'+1,'Y'+2]
	______________________________________________________________
	 |- ['Y'+1,'Y'+2]

  Moreover, a LaTeX-document is automatically generated and can be found in
  `invaxiomcalc/tex/InvAxiomCalc-Output.tex`

### Conventions ###

 1. An input axiom may consist of the following:
     - the letters [a-z] for atomic formulas
     - the constants `0`, `1`, `bot`, `top`
     - the logical connectives
        - `v` ... additive or (oplus)
        - `*` ... multiplicative and (otimes)
        - `&` ... additive and (&)
        - `+` ... multiplicative or (par)
        - `-` ... linear negation (nil)
 2. Examples of axioms are:
     - `a v (-a + -a + -a)`
     - `a + 1`
     - `-a + (a * a)`
     - `(-a + b) v (-b + a)`

 3. The algorithm works for axioms that belong to the classes N2 and P3' of
 the substructural hierarchy (see [1] for further details). If the provided
 axiom is not in N2 or P3', InvAxiomCalc only tells you the class where the
 axiom belongs to.

 4. The printed rules have the form:

          Premise_1   ... Premise_n
          __________________________
                Conclusion


   If the given axiom falls into class N2 or below, then a (set of) sequent rule(s)
   is generated where the premises and the conclusion consist of one component.
   Otherwise, if the axiom belongs to class P2 or P3' then a (set of) hypersequent
   rule(s) is generated where the premises and the conclusion consist of multiple
   components, e.g.:

     ?- compute.
     |: (-a + b) v (-b + a).

     This axiom is in the class: p(2)


     Equivalent Analytic Rule:

      H|  |- ['Y'+2,'Y'+3]      H|  |- ['Y'+4,'Y'+1]
     ______________________________________________________________
            H|  |- ['Y'+3,'Y'+4] |  |- ['Y'+1,'Y'+2]


   `'Y'+1`, `'Y'+2`, `'Y'+3` and `'Y'+4` are indexed fresh metavariables introduced during the completion
   procedure. They should be read as Gamma, Delta and Pi and Theta.

   The automatically generated LaTeX-document can be found in
      `invaxiomcalc/tex/InvAxiomCalc-Output.tex`

   The document contains the provided axiom, the automatically generated (set of)
   rule(s) and a suitable base calculus. Furthermore, some basic information regarding
   the new calculus and some theoretical results are provided in the document.

### About InvAxiomCalc ###

 InvAxiomCalc is the implementation of the procedure in
 
 [[1](http://www.logic.at/staff/agata/CSL09.pdf)] A. Ciabattoni, L. Strassburger, and K. Terui, Expanding the realm of systematic proof theory, Proceedings of Computer Science Logic, LNCS 5771, pp. 163--178, 2009.

 It takes as input any axiom in the language of a fragment of classical linear
 logic without exponentials (i.e. Multiplicative and Additive Linear Logic MALL) and:

  1.  identifies to which class in the substructural hierarchy of [1] the axiom
      belongs to.

  2. either of:
     * transforms each axiom within the class N2 into an equivalent structural
      sequent rule preserving cut-elimination when added MALL.

     * transforms each axiom within the class P3' into an equivalent structural
      hypersequent rule preserving cut-elimination when added to the hypersequent
      version of HMALL.

  3.  provides a paper containing an analytic calculus for (H)MALL extended with the
      given axiom.




Copyright by Ardit Ymeri, 2019.

Advisors: Agata Ciabattoni, Lara Katharina Spendier
