%% mall_grammar
%
% DCG for converting prolog representations to LaTeX.
%

bs       --> [92].
eol 	  	 --> [13,10].

texNewAxiom(C)			--> bs, "def", bs, "NewAxiom", "{$", C, "$}", eol.
texNewAxiomNotMath(C)	--> bs, "def", bs, "NewAxiomNotMath", "{", C, "}", eol. 
texNumCyclic(C)			--> bs, "def", bs, "NumCyclicPremises", "{$", C, "$}", eol. 
texNewRuleStart   		--> bs, "def", bs, "NewRule", "{".
texNewRuleEnd	  		--> "}", eol.
texNewNARuleStart  		--> bs, "def", bs, "NewNARule", "{".
texNewNARuleEnd    		--> "}", eol.
texAxiomClass(C)  		--> bs, "def", bs, "AxiomClass", "{$", bs, "mathcal", C, "$}", eol.
texNameBaseCalculus(C) 	--> bs, "def", bs, "NameBaseCalculus{",C,"}", eol.
texNameBaseCalculusP(C)	--> bs, "def", bs, "NameBaseCalculusP{",C,"}", eol.
texBaseCalculus(C) 		--> bs, "def", bs, "BaseCalculus{",bs,C,"}", eol.
texEquivalences(C) 		--> bs, "def", bs, "Equivalences{",bs,C,"}", eol.
texWrittenBC(C)	   		--> bs, "def", bs, "WrittenBC{",C,"}", eol.
texFullLambek(C)   		--> bs, "def", bs, "FullLambek{",bs,C,"}", eol.
texRule(C)				--> bs, "def", bs, "WRule{",C,"}", eol.
texRuleVerb(C)			--> bs, "def", bs, "WRuleV{",C,"}", eol.
texAbstract(C)			--> bs, "def", bs, "Abstract{",bs,C,"}", eol.
texTitle(C)				--> bs, "def", bs, "Title{",bs,C,"}", eol.
texSectionTitle(C)		--> bs, "def", bs, "SectionTitle{",bs,C,"}", eol.
acyclic 					--> bs, "def", bs, "Acyclicity{", bs,"Acyclic}", eol.
cyclic 					--> bs, "def", bs, "Acyclicity{", bs,"Cyclic}", eol.
defAcyclicity 			--> bs, "def", bs, "DefAcyclicity{", bs, "DefSequentAcyclicity}", eol.
defHyperAcyclicity 		--> bs, "def", bs, "DefAcyclicity{", bs, "DefHyperSequentAcyclicity}", eol.
axiomToSequentRule		--> bs, "def", bs, "FirstStransformationStep{", bs, "AxiomToSequentRule}", eol.
axiomToHypersequentRule	--> bs, "def", bs, "FirstStransformationStep{", bs, "AxiomToHypersequentRule}", eol.
texStructural(C)			--> bs, "def", bs, "WStructural{", C, "}", eol.

%\def\FirstStransformationStep{\AxiomToSequentRule}
%\def\WStructural{structural}

texNewAxC(C)		--> bs, "AxiomC{$",C,"$}",eol.
texNewUnC(C)		--> bs, "UnaryInfC{$",C,"$}",eol.
texNoLine		--> bs, "noLine", eol.

texNewUnCon(C)    --> bs, "UnaryInfC{$", C,"$}", 			bs, "DisplayProof", bs, "hspace*{0.3cm}", eol.
texNewBinCon(C)   --> bs, "BinaryInfC{$", C,"$}", 		bs, "DisplayProof", bs, "hspace*{0.3cm}", eol.
texNewTriCon(C)   --> bs, "TrinaryInfC{$", C,"$}", 		bs, "DisplayProof", bs, "hspace*{0.3cm}", eol.
texNewQuartCon(C) --> bs, "QuaternaryInfC{$", C, "$}", 	bs, "DisplayProof", bs, "hspace*{0.3cm}", eol.
texNewQuinCon(C)  --> bs, "QuinaryInfC{$", C, "$}",		bs, "DisplayProof", bs, "hspace*{0.3cm}", eol.

%rule2tex([[H|T],C])  --> rule2tex([H|T]),bs,"Ra",rule2tex(C).
rule2tex('G'+N)		--> bs, "Gamma_{",nr2tex(N),"}".
rule2tex('D'+N)		--> bs, "Delta_{",nr2tex(N),"}".
rule2tex('P'+N)		--> bs, "Pi_{",nr2tex(N),"}".
rule2tex('Y'+1)		--> bs, "Gamma".
rule2tex('Y'+2)		--> bs, "Delta".
rule2tex('Y'+3)		--> bs, "Sigma".
rule2tex('Y'+4)		--> bs, "Theta".
rule2tex('Y'+N)		--> bs, "Lambda_{",nr2tex(N),"}".
rule2tex(a)			--> " ", "A".
rule2tex(b)			--> " ", "B".
rule2tex(c)			--> " ", "C".
rule2tex(d)			--> " ", "D".
rule2tex(e)			--> " ", "E".
rule2tex(f)			--> " ", "F".
rule2tex(g)			--> " ", "G".
rule2tex(h)	 		--> " ", "H".
rule2tex(i)	 		--> " ", "I".
rule2tex(j)			--> " ", "J".
rule2tex(k)			--> " ", "K".
rule2tex(l)			--> " ", "L".
rule2tex(m)			--> " ", "M".
rule2tex(n)			--> " ", "N".
rule2tex(o)			--> " ", "O".
rule2tex(p)			--> " ", "P".
rule2tex(q)	  		--> " ", "Q".
rule2tex(r)	  		--> " ", "R".
rule2tex(s)	  		--> " ", "S".
rule2tex(t)	  		--> " ", "T".
rule2tex(u)	  		--> " ", "U".
rule2tex(w)	  		--> " ", "W".
rule2tex(x)	  		--> " ", "X".
rule2tex(y)	  		--> " ", "Y".
rule2tex(z)	  		--> " ", "Z".

rule2tex(-X)			--> rule2tex(X), "^{", axiom2tex(bot), "}".
rule2tex([])			--> "".

rule2tex([['HContex'] | T]) 	--> bs, "mathcal{H}", bs, ", | ", rule2tex(T).
rule2tex([[[], [H|T]] | []])	--> rule2tex([[],[H|T]]).
rule2tex([[[], [H|T]] | T2])	--> rule2tex([[],[H|T]]) , "|", rule2tex(T2).
rule2tex([[], [H|T]])		--> bs, "vdash", rule2tex([H|T]).
rule2tex([H | []])			--> rule2tex(H).
rule2tex([H | T])			--> rule2tex(H), ",", rule2tex(T).

nr2tex(N)	  --> {number_codes(N, Cs)}, Cs.

axiom2texP1(-X)    --> " ", axiom2texP1(X), "^", axiom2tex(bot).
axiom2texP1(X & Y) --> "(", axiom2texP2(X), " ", bs, "& ", axiom2texP2(Y), ")".
axiom2texP1(X v Y) --> "(", axiom2texP3(X), " ", bs, "oplus ", axiom2texP3(Y), ")".
axiom2texP1(X * Y) --> "(", axiom2texP4(X), " ", bs, "otimes ", axiom2texP4(Y), ")".
axiom2texP1(X + Y) --> "(", axiom2texP5(X), " ", bs, "nand ", axiom2texP5(Y), ")".
axiom2texP1(X)	   --> {atomic(X);member(X, [0, 1, bot, top])}, axiom2tex(X).

axiom2texP2(-X)    --> " ", axiom2texP1(X),  "^", axiom2tex(bot).
axiom2texP2(X & Y) --> axiom2texP2(X), " ", bs, "& ", axiom2texP2(Y).
axiom2texP2(X v Y) --> "(", axiom2texP3(X), " ", bs, "oplus ", axiom2texP3(Y), ")".
axiom2texP2(X * Y) --> "(", axiom2texP4(X), " ", bs, "otimes ", axiom2texP4(Y), ")".
axiom2texP2(X + Y) --> "(", axiom2texP5(X), " ", bs, "nand ", axiom2texP5(Y), ")".
axiom2texP2(X)	   --> {atomic(X);member(X, [0, 1, bot, top])}, axiom2tex(X).

axiom2texP3(-X)    --> " ", axiom2texP1(X), "^", axiom2tex(bot).
axiom2texP3(X & Y) --> "(", axiom2texP2(X), " ", bs, "& ", axiom2texP2(Y), ")".
axiom2texP3(X v Y) --> axiom2texP3(X), " ", bs, "oplus ", axiom2texP3(Y).
axiom2texP3(X * Y) --> "(", axiom2texP4(X), " ", bs, "otimes ", axiom2texP4(Y), ")".
axiom2texP3(X + Y) --> "(", axiom2texP5(X), " ", bs, "nand ", axiom2texP5(Y), ")".
axiom2texP3(X)	   --> {atomic(X);member(X, [0, 1, bot, top])}, axiom2tex(X).

axiom2texP4(-X)    --> " ", axiom2texP1(X), "^", axiom2tex(bot).
axiom2texP4(X & Y) --> "(", axiom2texP2(X), " ", bs, "& ", axiom2texP2(Y), ")".
axiom2texP4(X v Y) --> "(", axiom2texP3(X), " ", bs, "oplus ", axiom2texP3(Y), ")".
axiom2texP4(X * Y) --> axiom2texP4(X), " ", bs, "otimes ", axiom2texP4(Y).
axiom2texP4(X + Y) --> "(", axiom2texP5(X), " ", bs, "nand ", axiom2texP5(Y), ")".
axiom2texP4(X)	   --> {atomic(X);member(X, [0, 1, bot, top])}, axiom2tex(X).

axiom2texP5(-X)    --> " ", axiom2texP1(X), "^", axiom2tex(bot).
axiom2texP5(X & Y) --> axiom2texP2(X), " ", bs, "& ", axiom2texP2(Y).
axiom2texP5(X v Y) --> axiom2texP3(X), " ", bs, "oplus ", axiom2texP3(Y).
axiom2texP5(X * Y) --> axiom2texP4(X), " ", bs, "otimes ", axiom2texP4(Y).
axiom2texP5(X + Y) --> "(", axiom2texP5(X), " ", bs, "nand ", axiom2texP5(Y), ")".
axiom2texP5(X)	   --> {atomic(X);member(X, [0, 1, bot, top])}, axiom2tex(X).


axiom2tex(-X) 	 --> " ", axiom2texP1(X), "^", axiom2tex(bot).
axiom2tex(X & Y) --> axiom2texP2(X), " ", bs, "& ", axiom2texP2(Y).
axiom2tex(X v Y) --> axiom2texP3(X), " ", bs, "oplus ", axiom2texP3(Y).
axiom2tex(X * Y) --> axiom2texP4(X), " ", bs, "otimes ", axiom2texP4(Y).
axiom2tex(X + Y) --> axiom2texP5(X), " ", bs, "nand ", axiom2texP5(Y).
axiom2tex(1) 	--> "1".
axiom2tex(bot)	--> bs, "bot".
axiom2tex(top) 	--> bs, "top".
axiom2tex(0)	 	--> "0".
axiom2tex(a)		--> "A".
axiom2tex(b) 		--> "B".
axiom2tex(c)		--> "C".
axiom2tex(d)		--> "D".
axiom2tex(e)	 	--> "E".
axiom2tex(f)		--> "F".
axiom2tex(g)		--> "G".
axiom2tex(h)	 	--> "H".
axiom2tex(i)	 	--> "I".
axiom2tex(j)		--> "J".
axiom2tex(k)		--> "K".
axiom2tex(l)		--> "L".
axiom2tex(m)		--> "M".
axiom2tex(n)		--> "N".
axiom2tex(o)		--> "O".
axiom2tex(p)		--> "P".
axiom2tex(q)	  	--> "Q".
axiom2tex(r)	  	--> "R".
axiom2tex(s)	  	--> "S".
axiom2tex(t)	  	--> "T".
axiom2tex(u)	  	--> "U".
axiom2tex(w)	  	--> "W".
axiom2tex(x)	  	--> "X".
axiom2tex(y)	  	--> "Y".
axiom2tex(z)	  	--> "Z".


print_phrase(V) :-
   phrase(V, S),
   name(A, S),
   write(A).