:- use_module(library(time)).

:- ['invac_util.pl'].
:- ['invac_hierarchy.pl'].
:- ['invac_hyperstructural.pl'].
:- ['invac_analytic.pl'].
:- ['invac_acyclicity.pl'].
:- ['invac_printcmd.pl'].
:- ['invac_grammar.pl'].
:- ['invac_tex.pl'].
:- ['invac_axiom2rule.pl'].

%% mall_kernel
%
% The entry point of the program. Includes all the other modules. 
% Expects the input axiom from the user. Validates the input and 
% invokes the transformation procedure. 

%% compute
%
% Accepts the user input, validates it,  finds the corresponding class in the substructural
% hierarchy and invokes the transformation algorithm. 
compute :-
		catch(
		  read(Axiom),
		  Exception,
		  handle_read(Exception)
		),
  		(\+ axiom2tex(Axiom,_,[]) ->
		handle("1")
		; 
		is_in_class(Axiom, C),
		writef('\nThis axiom is in the class: %q\n', [C]),
        catch(
            call_with_time_limit(120, axiom_to_rule(Axiom, C)),
            Exception,                     
            handle_timeout(Exception))
		).

computeweb(Axiom) :-
  		(\+ axiom2tex(Axiom,_,[]) ->
		handle("1")
		; 
		is_in_class(Axiom, C),
		writef('\nThis axiom is in the class: %q\n', [C]),
        catch(
            call_with_time_limit(4, axiom_to_rule(Axiom, C)),
            Exception,                     
            handle_timeout(Exception))
		),
		halt.
		
handle_read(_) :-	
		tex_out(_, _, _, 100, 0),
		writef('\n\nThe axiom you provided is not computable.\n'),
		halt.

handle_timeout(_) :-	
		tex_out(_, _, _, 100, 0),
		writef('\n\nThe computation of the axiom timed out.\n'),
		halt.
		
handle(_) :-
        tex_out(_, _, _, 100, 0),
		writef('\n\nThe axiom you provided is not computable.'),
		halt.
		
