%% mall_printcmd
%
% Displays the generated rule into the console. 
%


%% print_rule(+Rule, +Class)
%			prints the rule on the commandline; if the rule is in the classes p(1), p(2) or
%			p(3), it is a hypersequent
print_rule([P, C], Class) :-
	nl, print_premises(P, Class,0), 
	writef('\n______________________________________________________________\n'),
	print_conclusion(C, Class).

%% print_premises(+Premise, +Class, +Counter)
%			prints the premises of the rule
print_premises([], _,_) :-
	write(' ').

print_premises([P|T1], Class, 2) :-
	write('\n'), 
	print_premise(P, Class),
	print_premises(T1, Class, 1).

print_premises([P|T1], Class, Counter) :-
	Counter1 is Counter+1,
	print_premise(P, Class),
	print_premises(T1, Class,Counter1).
	
print_premise([Premise | [] ], Class) :-
	member(Class, [n(1), n(2), p(1)]), 
	print_sequent(Premise),
	write('     ').

print_premise(Premise, Class) :-
	member(Class, [p(2), p_prime(3), p(3)]), 
	write('H'),
	print_hypersequent(Premise), 
	write('     ').
	
%% print_hypersequent(+Hypersequent)
%
print_hypersequent([]) :-
	write(' ').

print_hypersequent([H|T]) :-
	write('| '), 
	print_sequent(H),
	print_hypersequent(T).
	
print_hypersequent([H|[]]) :-
	print_sequent(H).
	
%% print_sequent(+Hypersequent)
%
print_sequent([]):-
	write(' ').

print_sequent([[], Succedent]) :-
	writef(' |- %p', [Succedent]).

%% print_conclusion(+Conclusion, Class)
%
print_conclusion([], _) :-
	write('  ').
	
print_conclusion(Conclusion, Class) :-
	member(Class, [p(2), p_prime(3), p(3)]),
	write('     H'),
	print_hypersequent(Conclusion).
	
print_conclusion([Conclusion | [] ], Class) :-
	member(Class, [n(1), n(2), p(1)]),
	print_sequent(Conclusion).
	
