%% mall_tex
%
% Prints in LaTeX format the related information that are needed for the generation of the article. 
%

tex_out(Axiom, ARules, NARules, C, NumCyclic) :-
	member(C, [p(1), n(1), n(2)]),
	axiom2tex(Axiom,TAxiom,[]),
	tell('tex/InvAxiomCalc.sty'), 
	length(ARules, N),
	constants2tex("MALL", N, C),
	print_phrase(texNewAxiom(TAxiom)),
	print_phrase(texNewAxiomNotMath(TAxiom)),
	create_cyclic_flag(NumCyclic),
	print_phrase(defAcyclicity),
	print_phrase(axiomToSequentRule),
	nl, print_phrase(texNewRuleStart),
	create_seq_rules(ARules),
	print_phrase(texNewRuleEnd),
	nl, print_phrase(texNewNARuleStart),
	create_seq_rules(NARules),
	print_phrase(texNewNARuleEnd),
	told.
	
tex_out(Axiom, ARules, NARules, C, NumCyclic) :-
	member(C, [p(2), p_prime(3)]),
	axiom2tex(Axiom,TAxiom,[]),
	tell('tex/InvAxiomCalc.sty'), 
	length(ARules, N),
	%we dont need weakening for classes up to p_prime(3)
	constants2tex("HMALL", N, C),
	print_phrase(texNewAxiom(TAxiom)),
	print_phrase(texNewAxiomNotMath(TAxiom)),
	create_cyclic_flag(NumCyclic),
	print_phrase(defHyperAcyclicity),
	print_phrase(axiomToHypersequentRule),
	nl, print_phrase(texNewRuleStart),
	create_hseq_rules(ARules),
	print_phrase(texNewRuleEnd),
	nl, print_phrase(texNewNARuleStart),
	create_hseq_rules(NARules),
	print_phrase(texNewNARuleEnd),
	told.

tex_out(Axiom, ARules, NARules, C, NumCyclic) :-
	member(C, [p(3)]),
	axiom2tex(Axiom,TAxiom,[]),
	tell('tex/InvAxiomCalc.sty'),
	length(ARules, N),
	constants2tex("HMALLw", N, C),
	print_phrase(texNewAxiom(TAxiom)),
	print_phrase(texNewAxiomNotMath(TAxiom)),
	create_cyclic_flag(NumCyclic),
	print_phrase(defHyperAcyclicity),
	print_phrase(axiomToHypersequentRule),
	nl, print_phrase(texNewRuleStart),
	create_hseq_rules(ARules),
	print_phrase(texNewRuleEnd),
	nl, print_phrase(texNewNARuleStart),
	create_hseq_rules(NARules),
	print_phrase(texNewNARuleEnd),
	told.

tex_out(_, _, _, C, _) :-
	\+ member(C, [p(1), p(2), p_prime(3), p(3), n(1), n(2)]),
	tell('tex/InvAxiomCalc.sty'),
	write('No'),
	told.
	
create_cyclic_flag(NumCyclic):- 
	NumCyclic = 0,
	print_phrase(acyclic).

create_cyclic_flag(NumCyclic):- 
	NumCyclic \= 0,
	print_phrase(cyclic).

%% constants2tex(+BaseCalc, +NumRules, +Class)
% 
% Prints the constatns related to the base calculus. 
%
% +BaseCalc 		- either MALL or HMALL
% +NumRules			- the number of generated rules
% +Class			- the class in substructural hierarchy of the input axiom
%
constants2tex("MALL", N, _) :-
	( (N = 1) ->
	  print_phrase(texRule("rule"))
 	; print_phrase(texRule("rules"))
	),
	print_phrase(texAxiomClass("{N}_2")),
	print_phrase(texWrittenBC("sequent")),
	print_phrase(texNameBaseCalculus("MALL")),
	print_phrase(texNameBaseCalculusP("MALL")),
	print_phrase(texBaseCalculus("CalculusMall")),
	print_phrase(texEquivalences("EquivalenceSeq")),
	print_phrase(texStructural("structural")).

constants2tex("HMALL", N, C) :-
	( N = 1 ->
	  print_phrase(texRule("rule"))
 	; print_phrase(texRule("rules"))
	),
	( member(C, [p(2)]) ->
	  print_phrase(texAxiomClass("{P}_2"))
	; member(C, [p_prime(3)]) ->
	  	print_phrase(texAxiomClass("{P}^{'}_3"))
	),
	print_phrase(texWrittenBC("hypersequent")),
	print_phrase(texNameBaseCalculus("HMALL")),
	print_phrase(texNameBaseCalculusP("HMALL")),
	print_phrase(texBaseCalculus("CalculusHMall")),
	print_phrase(texEquivalences("EquivalenceHSeq")), 
	print_phrase(texStructural("hyperstructural")).
	
constants2tex("HMALLw", N, C) :-
	( N = 1 ->
	  print_phrase(texRule("rule"))
 	; print_phrase(texRule("rules"))
	),
	( member(C, [p(3)]) ->
	  print_phrase(texAxiomClass("{P}_3"))
	),
	print_phrase(texWrittenBC("hypersequent")),
	print_phrase(texNameBaseCalculus("HMALLw")),
	print_phrase(texNameBaseCalculusP("MALLw")),
	print_phrase(texBaseCalculus("CalculusHMallw")),
	print_phrase(texEquivalences("EquivalenceHSeq")),
	print_phrase(texStructural("hyperstructural")).

constants2tex("MTL", N, C) :-
	( N = 1 ->
	  print_phrase(texRule("rule")),
	  print_phrase(texRuleVerb("is"))
 	; print_phrase(texRule("rules")),
	  print_phrase(texRuleVerb("are"))
	),
	( member(C, [p(2)]) ->
	  print_phrase(texAxiomClass("{P}_2"))
	; ( member(C, [n(2)]) ->
	    print_phrase(texAxiomClass("{N}_2"))
	  ; print_phrase(texAxiomClass("{P}_3"))
	  )
	).

%% create_seq_rule(+Rules)
% Rules - list of analytic sequent rules
create_seq_rules([]).
create_seq_rules([[Prem, Con]|T]) :-
	create_seq_rule(Prem, Con),
	create_seq_rules(T).

%% create_hseq_rules(+Rules)
% Rules - list of analytic hypersequent rules
create_hseq_rules([]).
create_hseq_rules([[Prem, Con]|T]) :-
	create_hseq_rule(Prem, Con),
	create_hseq_rules(T).

%% create_seq_rule(+Prem, +Con)
% Prem - list of premises of the rule
% Con - conclusion of the rule
create_seq_rule(Prem, Con) :-
	length(Prem,N),
	member(N, [0,1]),
	rule2tex(Prem, TPrem, []),
	rule2tex(Con, TCon, []),
	print_phrase(texNewAxC(TPrem)),
	print_phrase(texNewUnCon(TCon)).

create_seq_rule(Prem, Con) :-
	length(Prem,N),
	member(N, [2,4]),
	createPremises(Prem, TPrem),
	N1 is N/2+1,
	printBinPremises(0, TPrem, N1, 1),
	rule2tex(Con, TCon, []),
	print_phrase(texNewBinCon(TCon)).

create_seq_rule(Prem, Con) :-
	length(Prem,N),
	createPremises(Prem, TPrem),
	N1 is N//3,
	N2 is N mod 3,
	printTriPremises(0, TPrem, N1, N2, 1),
	rule2tex(Con, TCon, []),
	print_phrase(texNewTriCon(TCon)).

%% create_hseq_rule(+Prem, +Con)
% 	Prem,Con  ... Analytic Hypersequent Rule for the axiom (Premises, Conclusion)
create_hseq_rule(Prem, Con) :-
	length(Prem,N),
	member(N, [0,1]),
	create_hseq_premises(Prem, TPrem1),
	append([],TPrem1,TPrem),
	create_hseq_conclusion(Con, TCon1),
	append([],TCon1,TCon),
	nl,
	print_phrase(texNewAxC(TPrem)),
	print_phrase(texNewUnCon(TCon)).

create_hseq_rule(Prem, Con) :-
	length(Prem,N),
	member(N, [2,4]),
	create_hseq_premises(Prem, TPrem),
	nl,
	N1 is N/2+1,
	printBinPremises(1, TPrem, N1, 1),
	create_hseq_conclusion(Con, TCon1),
	append([],TCon1,TCon),
	print_phrase(texNewBinCon(TCon)).

create_hseq_rule(Prem, Con) :-
	length(Prem,N),
	create_hseq_premises(Prem, TPrem),
	nl,
	N1 is N//3,
	N2 is N mod 3,
	printTriPremises(1, TPrem, N1, N2, 1),
	create_hseq_conclusion(Con, TCon1),
	append([],TCon1,TCon),
	print_phrase(texNewTriCon(TCon)).

%% create_hseq_conclusion(+Con,-TCon])
% 	Con  ... Conclusion of the rule computed by AxiomCalc
%	TCon ... Conclusion of the rule in tex-format
%
create_hseq_conclusion(C, TCon) :-
	append([['HContex']], C, HCon), 
	hseq_tex_conclusion(HCon, TCon).

hseq_tex_conclusion([], []).	
hseq_tex_conclusion(C, TH):-	
	rule2tex(C, TH, []).

%% createPremises(+Prem,-TPrem])
% 
% +Prem   - Premises of the rule computed by InvAxiomCalc
% -TPrem  - Premises of the rule in tex-format
createPremises([], []).
createPremises([H|T], TPrem) :-
	rule2tex(H, TH, []),
	createPremises(T, TPrem1),
	append([TH],TPrem1,TPrem).
	
%% create_hseq_premises(+HPremises, -TPremises)
%
% +HPremises 	- Hypersequent premises generated by InvAxiomCalc
% -TPremises	- Hypersequent premises in tex-format
%
create_hseq_premises([], []).
create_hseq_premises([H|T], TPrem) :-
	append([['HContex']], H, H1),
	rule2tex(H1, TH, []),
	create_hseq_premises(T, TPrem1),
	append([TH],TPrem1,TPrem).

%% printBinPremises(+Hyper, +Prem, +N, +I])
%	for (hyper)sequent rules that have 2 or 4 premises
% 	Hyper ... is 1 if hypersequent, 0 if sequent
%	Prem  ... Premises of the rule
%	N     ... number of premises
%	I     ... counter
printBinPremises(_, [], _, _).

printBinPremises(Hyper, [H|T], N, I) :-
	member(I, [1,N]),
	I1 is I+1,
	( Hyper = 1 ->
	  append([],H,H1)
	; H1 = H
	),
	print_phrase(texNewAxC(H1)),
	printBinPremises(Hyper, T, N, I1).
	
printBinPremises(Hyper, [H|T], N, I) :-
	I1 is I+1,
	( Hyper = 1 ->
	  append([],H,H1)
	; H1 = H
	),
	print_phrase(texNoLine),
	print_phrase(texNewUnC(H1)),
	printBinPremises(Hyper, T, N, I1).

%% printTriPremises(+Hyper, +Prem, +Div, +Mod, +I])
%	for (hyper)sequent rules that have 3 or >4 premises
%
% 	Hyper ... is 1 if hypersequent, 0 if sequent
%	Prem  ... Premises of the rule
%	Div   ... number of premises / 3 (Integer division)
%	Mod   ... number of premises mod 3
%		  --> to balance the number of premises in the rule!
%	I     ... counter
printTriPremises(_, [], _, _, _).

printTriPremises(Hyper, [H|T], Div, Mod, I) :-
	( Mod > 0 ->
 	  M1 is 1+Div+1
	; M1 is 1+Div
	), 
	( Mod > 1 ->
	  M2 is M1+Div+1
	; M2 is M1+Div
	),
	member(I, [1,M1,M2]),
	I1 is I+1,
	( Hyper = 1 ->
	  %append(["G\\hh"],H,H1)
	  append([],H,H1)
	; H1 = H
	),
	print_phrase(texNewAxC(H1)),
	printTriPremises(Hyper, T, Div, Mod, I1).
	
printTriPremises(Hyper, [H|T], Div, Mod, I) :-
	I1 is I+1,
	( Hyper = 1 ->
	  %append(["G\\hh"],H,H1)
	  append([],H,H1)
	; H1 = H
	),
	print_phrase(texNoLine),
	print_phrase(texNewUnC(H1)),
	printTriPremises(Hyper, T, Div, Mod, I1).

