%% mall_util
%
% A set of helper predicates and  that are needed in the implementation
% of the transformation algorithm. 
%

:- op(475, yfx, v).      	% disjunction
:- op(450, yfx, &).	  		% conjunction
	
%% is_atom(+Axiom)
%
% checks wether the given axiom is an atom. Note that the 
% constant symbols 0, 1, 'top' and 'bot' are excluded.
%
is_atom(A) :- 
	atomic(A), 
	\+ member(A, [0, 1, bot, top]).
	
%% remove(+Element, +List, -List)
%
% Removes an element from a list
remove(X, L2, L3) :-
	\+ member(X, L2),
	append([], L2, L3).

remove(H, [H|L2], L3) :-
	append(L2, [], L3).

remove(X, [H|L2], L3) :-
	X \= H,
	member(X, L2),
	remove(X, L2, L4),
	append([H], L4, L3).
	

%% removeAll(+Element, +List, -List)
%
% Removes ALL elements Element from a list
removeAll(X, L2, L3) :-
	\+ member(X, L2),
	append([], L2, L3).

removeAll(H, [H|L2], L3) :-
	removeAll(H, L2, L4),
	append(L4, [], L3).

removeAll(X, [H|L2], L3) :-
	X \= H,
	member(X, L2),
	removeAll(X, L2, L4),
	append([H], L4, L3).

%  reverseList
%		reverses the list
reverseList([],A,A).
reverseList([H|T],A,R) :- reverseList(T,[H|A],R).


%% count_repetitions(+Sequent, +Mv, -Count)
%
% Counts the number of occurrences of a metavariable in a sequent. 
%
% +Sequent 		- a sequent to be searched for occurrences of MV
% +Mv			- a metavariable to b e counted
% +Count		- the number of repetitions of MV in the given Sequent.
%
count_repetitions([], _, 0).

count_repetitions([H|T], Mv, NumRep) :-
	count_repetitions(T, Mv, NumRep1),
	(H = Mv -> 
		NumRep is NumRep1 + 1
	   ;NumRep is NumRep1).