
:- ['../util.pl'].
:- ['../hierarchy.pl'].

:- begin_tests(hierarchy).

test(is_pos_class) :-
	is_pos_class(a v (-a + -a + -a), 0, 2).

% Nelson axiom - expected least positive class: 3	
test(is_pos_class) :-
	is_pos_class( ( ((a * a * b) v (b * b * a) ) + -a + -b ), 0, 3).

% Linearity axiom - expected least positive class: 3	
test(is_pos_class) :-
	is_pos_class((-a + b) & 1 v (-b + a) & 1, 0, 3).
	
% Excluded middle - expected least positive class: 1
test(is_pos_class) :-
	is_pos_class(a v -a, 0, 1).
	
% Contraction axiom - expected least positive class: 3	
test(is_pos_class) :-
	is_pos_class(-a + (a * a), 0, 3).

% Weakening axiom - expected least positive class: 3
test(is_pos_class) :-
	is_pos_class(a + 1, 0, 3).
	
	
%%
% testing is_pos_axiom()
%

test(is_pos_axiom) :-
	is_pos_axiom(a, 1).

test(is_pos_axiom) :-
	is_pos_axiom(a, 0).

test(is_pos_axiom) :-
	is_pos_axiom(a + 1, 3).	

test(is_pos_axiom, fail) :-
	is_pos_axiom(a + 1, 2).	

test(is_pos_axiom) :-
	is_pos_axiom(a v -a, 1).	
	
%%
% testing is_in_neg_class
%

% n-em - P2, N3
test(is_neg_class) :-
	is_neg_class(a v (-a + -a + -a), 0, 3).

% nelseon - N2 		is_neg_class( ( ((a * a * b) v (b * b * a) ) + -a + -b ), 0, N).
test(is_neg_class) :-
	is_neg_class(( ((a * a * b) v (b * b * a) ) + -a + -b ), 0, 2).

% linearity - P'3, P3, N4 
test(is_neg_class) :-
	is_neg_class((-a + b) & 1 v (-b + a) & 1 , 0, 4).
	
% w_linearity_w - P2, N3 
test(is_neg_class) :-
	is_neg_class((-a + b) v (-b + a) , 0, 3).
	
% em: 			is_neg_class(a v -a, 0, N).
test(is_neg_class) :-
	is_neg_class((-a + b) v (-b + a) , 0, 3).

test(is_neg_class, fail) :-
	is_neg_class((-a + b) v (-b + a) , 0, 2).
	
% contraction:	N2
test(is_neg_class) :-
	is_neg_class(-a + (a * a), 0, 2).
	
% weakening: N2
test(is_neg_class) :-
	is_neg_class(a + 1 , 0, 2).
	
%%
% Testing is_negative_axiom/2
%

% nelseon - N2 		is_neg_class( ( ((a * a * b) v (b * b * a) ) + -a + -b ), 0, N).
test(is_neg_axiom) :-
	is_neg_axiom((((a * a * b) v (b * b * a) ) + -a + -b ), 2).

% linearity - P'3, P3, N4 
test(is_neg_axiom) :-
	is_neg_axiom((-a + b) & 1 v (-b + a) & 1, 4).
	
% w_linearity_w - P2, N3 
test(is_neg_axiom) :-
	is_neg_axiom((-a + b) v (-b + a), 3).
	
% em: 			is_neg_class(a v -a, 0, N).
test(is_neg_axiom) :-
	is_neg_axiom((-a + b) v (-b + a), 3).

test(is_neg_axiom, fail) :-
	is_neg_axiom((-a + b) v (-b + a), 2).
	
% contraction:	N2
test(is_neg_axiom) :-
	is_neg_axiom(-a + (a * a), 2).
	
% weakening: N2
test(is_neg_axiom) :-
	is_neg_axiom(a + 1, 2).
	
	
%%
% Testing is_pos3_prime/1

test(is_pos_3prime_axiom) :-
	is_pos_3prime_axiom(1).

test(is_pos_3prime_axiom) :-
	is_pos_3prime_axiom(0).

test(is_pos_3prime_axiom) :-
	is_pos_3prime_axiom((-b + a) & 1).

test(is_pos_3prime_axiom) :-
	is_pos_3prime_axiom((-a + b) & 1 v (-b + a) & 1).	

test(is_pos_3prime_axiom) :-
	is_pos_3prime_axiom(((-a + b) & 1) * ((-b + a) & 1)).	
	
%%
% testing is_in_class
%

% 3-em - P2
test(is_in_class) :- 
	is_in_class(a v (-a + -a + -a), p(2)).

%Nelson - N2	
test(is_in_class) :- 
	is_in_class(( ((a * a * b) v (b * b * a) ) + -a + -b ), n(2)).

% linearity - P'3	
test(is_in_class) :- 
	is_in_class((-a + b) & 1 v (-b + a) & 1, p_prime(3)).

% em - P1
test(is_in_class) :- 
	is_in_class(a v -a, p(1)).
	
% contraction: N2
test(is_in_class) :- 
	is_in_class(-a + (a * a), n(2)).
	
% weakening - N2
test(is_in_class) :- 
	is_in_class(a + 1, n(2)).
		
:- end_tests(hierarchy).
	


%tests: 
% n-em : 		a v (-a + -a + -a)
% nelseon: 		((a * a * b) v (b * b * a) ) + -a + -b
% linearity:	 	(-a + b) & 1 v (-b + a) & 1
% w_linearity:	(-a + b) v (-b + a)  
% em: 			a v -a, 0, N).
% contraction:	-a + (a * a)
% weakening: 	a + 1

%tests: 
% n-em : 		is_pos_class(a v (-a + -a + -a), 0, P). 
% nelseon: 		is_pos_class( ( ((a * a * b) v (b * b * a) ) + -a + -b ), 0, P).
% linearity:	is_pos_class( (-a + b) & 1 v (-b + a) & 1 , 0, P).
% linearity_':	is_pos_class( (-a + b) v (-b + a) , 0, P). 
% em: 			is_pos_class(a v -a, 0, P).
% contraction:	is_pos_class(-a + (a * a), 0, P).
% weakening: 	is_pos_class(a + 1, 0, P).

% n-em : 		is_neg_class(a v (-a + -a + -a), 0, N). 
% nelseon: 		is_neg_class( ( ((a * a * b) v (b * b * a) ) + -a + -b ), 0, N).
% linearity:	is_neg_class( (-a + b) & 1 v (-b + a) & 1 , 0, N).
% linearity_':	is_neg_class( (-a + b) v (-b + a) , p(2), 0, N). 
% em: 			is_neg_class(a v -a, 0, N).
% contraction:	is_neg_class(-a + (a * a), 0, N).
% weakening: 	is_neg_class(a + 1, 0, N).

% n-em : 		is_neg_class(a v (-a + -a + -a), 0, N). 
% nelseon: 		is_neg_class( ( ((a * a * b) v (b * b * a) ) + -a + -b ), 0, N).
% linearity:	is_neg_class( (-a + b) & 1 v (-b + a) & 1 , 0, N).
% linearity_':	is_neg_class( (-a + b) v (-b + a) , 0, N). 
% em: 			is_neg_class(a v -a, 0, N).
% contraction:	is_neg_class(-a + (a * a), 0, N).
% weakening: 	is_neg_class(a + 1, 0, N).


% n-em : p2			is_in_class(a v (-a + -a + -a), C). 
% nelseon: n2		is_in_class( ( ((a * a * b) v (b * b * a) ) + -a + -b ), C).
% linearity: p'(3)	is_in_class( (-a + b) & 1 v (-b + a) & 1 , C).
% em: p(1)			is_in_class(a v -a, C).
% contraction: n(2)	is_in_class(-a + (a * a), C).
% weakening: n(2) 	is_in_class(a + 1, C).

