:- ['../util'].
:- ['../hyperstructural'].

:- begin_tests(hyperstructural).

%%
% Testing aths_ms_r/3
%

test(aths_ms_r) :-
	aths_ms_r(a + b + c, 
	n(1), 
	[[[[[], ['Y'+1, -a]]], [[[], ['Y'+2, -b]]], [[[], ['Y'+3, -c]]]], [[[], ['Y'+1, 'Y'+2, 'Y'+3]]]]).

test(aths_ms_r) :-
	aths_ms_r((a + b) + (c + d), 
	n(1), 
	[[[[[], ['Y'+1, -a]]], [[[], ['Y'+2, -b]]], [[[], ['Y'+3, -c]]], [[[], ['Y'+4, -d]]]], [[[], ['Y'+1, 'Y'+2, 'Y'+3, 'Y'+4]]]]).
	
test(aths_ms_r) :-
	aths_ms_r((a + bot) + (c + d), 
	n(1), 
	[[[[[], ['Y'+1, -c]]], [[[], ['Y'+2, -d]]], [[[], ['Y'+3, -a]]]], [[[], ['Y'+1, 'Y'+2, 'Y'+3]]]]).
	
% aths_ms_r((a + top) * (c + d), n(1), X).  FIXME: Issue #7
% test(aths_ms_r) :- 
%	aths_ms_r((a + top) * (c + d), 
%	n(1), 
%	[[[[[], [-a*top, -c* -d, 'Y'+1]]]], [[[], ['Y'+1]]]]).

test(aths_ms_r) :-
	aths_ms_r( ( ((a * a * b) v (b * b * a) ) + -a + -b ), 
	n(2), 
	[[[[[], [-b, -b, -a, 'Y'+1]]], [[[], [-a, -a, -b, 'Y'+1]]], [[[], ['Y'+2, a]]], [[[], ['Y'+3, b]]]], [[[], ['Y'+1, 'Y'+2, 'Y'+3]]]]).
	
%%
% Testing apply_de_morgan/2
%

test(apply_de_morgan) :-
	apply_de_morgan(-( a v b ), -a & -b).
	
test(apply_de_morgan) :-
	apply_de_morgan(-( a & b ), -a v -b).
	
test(apply_de_morgan) :-
	apply_de_morgan(-( a * b ), -a + -b).

test(apply_de_morgan) :-
	apply_de_morgan(-( a + b ), -a * -b).
	
test(apply_de_morgan) :-
	apply_de_morgan(-( (a v c) + b ), (-a & -c) * -b).
	
test(apply_de_morgan) :-
	apply_de_morgan(-(a), -a).
	
test(apply_de_morgan) :-
	apply_de_morgan(-(0), 1).
	
test(apply_de_morgan) :-
	apply_de_morgan(-(1), 0).
	
test(apply_de_morgan) :-
	apply_de_morgan(-(top), bot).
	
test(apply_de_morgan) :-
	apply_de_morgan(-(bot), top).
	
test(apply_de_morgan) :-
	apply_de_morgan(a + b, a + b).
	

%%
% Testing apply_to_conclusion/2
%
test(apply_to_conclusion) :-
	apply_to_conclusion([[[],[a + b + c]]], [[[],[a, b, c]]]).
	
test(apply_to_conclusion) :-
	apply_to_conclusion([[[],[((a * a * b) v (b * b * a) ) + -a + -b]]], [[[],[((a * a * b) v (b * b * a) ), -a, -b]]]).

test(apply_to_conclusion) :-
	apply_to_conclusion([[[],[a v (-a + -a + -a)]]], [[[],[a v (-a + -a + -a)]]]).

test(apply_to_conclusion) :-
	apply_to_conclusion([[[],[ -a + (a * a) ]]], [[[],[ -a, (a * a) ]]]).

test(apply_to_conclusion) :-
	apply_to_conclusion([[[],[ (-a + b) v (-b + a) ]]], [[[],[ (-a + b) v (-b + a) ]]]).
	

test(apply_to_conclusion) :-
	apply_to_conclusion([[[],[ a v -a ]]], [[[],[ a v -a ]]]).
	

test(apply_to_conclusion) :-
	apply_to_conclusion([[[],[ a + 1 ]]], [[[],[ a, 1 ]]]).

%%
% Testing apply_equivalence_lemma/5
%

test(apply_equivalence_lemma) :-
	apply_equivalence_lemma(
	[], 
	[[[[], ['Y'+1,  -a ]]], [[[], ['Y'+2, 0]]]], 
	[[[],[a , 1]]], 
	[[[], ['Y'+1, 'Y'+2]]], 
	3).

test(apply_equivalence_lemma) :-
	apply_equivalence_lemma(
	[], 
	[[[[], ['Y'+1,  -a & (a * a * a)]]]], 
	[[[],[a v (-a + -a + -a)]]], 
	[[[], ['Y'+1]]], 
	2).
	
test(apply_equivalence_lemma) :-
	apply_equivalence_lemma(
	[], 
	[[[[], ['Y'+1,  (-a + -a + -b)&(-b + -b + -a)]]], [[[], ['Y'+2, a]]], [[[], ['Y'+3, b]]]], 
	[[[],[((a * a * b) v (b * b * a) ), -a, -b]]], 
	[[[], ['Y'+1, 'Y'+2, 'Y'+3]]], 
	4).

%%
% Testing apply_invertible_to_premises/2
%

%test(apply_invertible_to_premises) :-  FIXME ISSUE #8
%	apply_invertible_to_premises(
%	[[[[], ['Y'+1,  (-a + -a + -b)&(-b + -b + -a)]]], [[[], ['Y'+2, a]]], [[[], ['Y'+3, b]]]],
%	[ [[[], ['Y'+1, -a, -a, -b]]], [[[], ['Y'+1, -b, -b, -a]]], [[[], ['Y'+2, a]]], [[[], ['Y'+3, b]]]]
%	).


%test(apply_invertible_to_premises) :-  FIXME ISSUE #8
%	apply_invertible_to_premises(
%	[[[[], ['Y'+1,  -a & (a * a * a)]]]],
%	[[[[], [a*a*a, 'Y'+1]]], [[[], [-a, 'Y'+1]]]]
%	).


% n-em : 		a v (-a + -a + -a)
% nelseon: 		((a * a * b) v (b * b * a) ) + -a + -b
% linearity:	 	(-a + b) & 1 v (-b + a) & 1
% w_linearity:	(-a + b) v (-b + a)  
% em: 			a v -a
% contraction:	-a + (a * a)
% weakening: 	a + 1

:- end_tests(hyperstructural).


 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % some tests:
 
 % aths_ms_r(a + b + c, n(1), X).
 % aths_ms_r((a + b) + (c + d), n(1), X).
 % aths_ms_r((a + bot) + (c + d), n(1), X).
 % aths_ms_r((a + top) + (c + d), n(1), X).
 % aths_ms_r((a + top) + (c + d), n(1), X).
 
 % aths_ms_r( ( ((a * a * b) v (b * b * a) ) + -a + -b ), n(1), X).  Nelseon Axiom

 
 % apply_de_morgan(-(a * (c & d)), X).